<?php
abstract class Model
{
	private $db;	
	private $stmt;
	private $error;

	public function __construct()
	{		
		try 
		{
			// Set DSN
			$dsn = DB_TYPE.":charset=utf8;host=".DB_HOST.";dbname=".DB_NAME;
			// Set Options
			$options = array(
				PDO::ATTR_PERSISTENT	=> true,
				PDO::ATTR_ERRMODE		=> PDO::ERRMODE_EXCEPTION
			);
			// Create new PDO
			$this->db = new PDO($dsn, DB_USER, DB_PASS, $options);
		}
		catch (PDOException $e)
		{
			$this->error = $e->getMessage();
		}		
	}

	public static function showError()
	{
		return $this->error;
	}

	public function query($query)
	{
		$this->stmt = $this->db->prepare($query);
	}

	//Binds the prep statement
	public function bind($param, $value, $type = null){
 		if (is_null($type)) {
  			switch (true) {
    			case is_int($value):
      				$type = PDO::PARAM_INT;
      				break;
    			case is_bool($value):
      				$type = PDO::PARAM_BOOL;
      				break;
    			case is_null($value):
      				$type = PDO::PARAM_NULL;
      				break;
    			default:
      				$type = PDO::PARAM_STR;
  			}
		}
		$this->stmt->bindValue($param, $value, $type);
	}

	public function execute()
	{	
		return $this->stmt->execute();
	}

	public function close() 
	{
		return $this->db = null;
	}

	public function open() 
	{
		// If successful  
		if ($this->db)
		{  
			// Return True  
			return true;  
		}  
		// Return False  
		return false;  
	}

	public function lastInsertId() 
	{
		return $this->db->lastInsertId();
	}

	public function single() 
	{
		$this->stmt->execute();
		return $this->stmt->fetch(PDO::FETCH_ASSOC);
	}

	public function resultSet()
	{
		$this->stmt->execute();
		return $this->stmt->fetchAll(PDO::FETCH_ASSOC);
	}

	public function rowCount()
	{
		return $this->stmt->rowCount();
	}
}